#!/usr/bin/env python
from setuptools import setup
#from distutils.core import setup

setup(
	name='linguini',
	version='1.1.6',
	description='beats both spaghetti and ravioli',
	author='edward newell',
	author_email='edward.newell@gmail.com',
	packages=['linguini'],
	scripts=['bin/linguini'],
	license='MIT',
	classifiers=[
		'Development Status :: 2 - Pre-Alpha',
		'Intended Audience :: Science/Research',
		'Intended Audience :: System Administrators',
		'Intended Audience :: Information Technology',
		'Programming Language :: Python :: 2.7',
		'Natural Language :: English',
		'Topic :: Utilities',
	],
	keywords='pipe dream'
)

